﻿using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Win8Contracts.Common;

using System;
using Win8Contracts.Data;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.Search;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

// The Grid App template is documented at http://go.microsoft.com/fwlink/?LinkId=234226

namespace Win8Contracts
{
	/// <summary>
	/// Provides application-specific behavior to supplement the default Application class.
	/// </summary>
	sealed partial class App : Application
	{
		/// <summary>
		/// Initializes the singleton Application object.  This is the first line of authored code
		/// executed, and as such is the logical equivalent of main() or WinMain().
		/// </summary>
		public App()
		{
			this.InitializeComponent();
			this.Suspending += OnSuspending;
		}

		/// <summary>
		/// Invoked when the application is launched normally by the end user.  Other entry points
		/// will be used when the application is launched to open a specific file, to display
		/// search results, and so forth.
		/// </summary>
		/// <param name="args">Details about the launch request and process.</param>
		protected override async void OnLaunched(LaunchActivatedEventArgs args)
		{
			await SampleDataSource.LoadAll();

			Frame rootFrame = Window.Current.Content as Frame;

			// Do not repeat app initialization when the Window already has content,
			// just ensure that the window is active
			
			if (rootFrame == null)
			{
				// Create a Frame to act as the navigation context and navigate to the first page
				rootFrame = new Frame();
				//Associate the frame with a SuspensionManager key                                
				SuspensionManager.RegisterFrame(rootFrame, "AppFrame");

				if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
				{
					// Restore the saved session state only when appropriate
					try
					{
						await SuspensionManager.RestoreAsync();
					}
					catch (SuspensionManagerException)
					{
						//Something went wrong restoring state.
						//Assume there is no state and continue
					}
				}

				// Place the frame in the current Window
				Window.Current.Content = rootFrame;
			}
			if (rootFrame.Content == null)
			{
				// When the navigation stack isn't restored navigate to the first page,
				// configuring the new page by passing required information as a navigation
				// parameter
				if (!rootFrame.Navigate(typeof(GroupedItemsPage), "AllGroups"))
				{
					throw new Exception("Failed to create initial page");
				}
			}
			// Ensure the current window is active
			Window.Current.Activate();
		}

		/// <summary>
		/// Invoked when application execution is being suspended.  Application state is saved
		/// without knowing whether the application will be terminated or resumed with the contents
		/// of memory still intact.
		/// </summary>
		/// <param name="sender">The source of the suspend request.</param>
		/// <param name="e">Details about the suspend request.</param>
		private async void OnSuspending(object sender, SuspendingEventArgs e)
		{
			var deferral = e.SuspendingOperation.GetDeferral();
			await SuspensionManager.SaveAsync();
			deferral.Complete();
		}

		/// <summary>
		/// Invoked when the application is activated as the target of a sharing operation.
		/// </summary>
		/// <param name="args">Details about the activation request.</param>
		protected override void OnShareTargetActivated(ShareTargetActivatedEventArgs args)
		{
			var shareTargetPage = new ShareTargetPage();
			shareTargetPage.Activate(args);
		}

		/// <summary>
		/// Invoked when the application is activated to display search results.
		/// </summary>
		/// <param name="args">Details about the activation request.</param>
		protected async override void OnSearchActivated(SearchActivatedEventArgs args)
		{
			// TODO: Register the Windows.ApplicationModel.Search.SearchPane.GetForCurrentView().QuerySubmitted
			// event in OnWindowCreated to speed up searches once the application is already running

			// If the Window isn't already using Frame navigation, insert our own Frame
			var previousContent = Window.Current.Content;
			var frame = previousContent as Frame;

			// If the app does not contain a top-level frame, it is possible that this 
			// is the initial launch of the app. Typically this method and OnLaunched 
			// in App.xaml.cs can call a common method.
			if (frame == null)
			{
				// Create a Frame to act as the navigation context and associate it with
				// a SuspensionManager key
				frame = new Frame();
				Win8Contracts.Common.SuspensionManager.RegisterFrame(frame, "AppFrame");

				if (args.PreviousExecutionState == ApplicationExecutionState.Terminated)
				{
					// Restore the saved session state only when appropriate
					try
					{
						await Win8Contracts.Common.SuspensionManager.RestoreAsync();
					}
					catch (Win8Contracts.Common.SuspensionManagerException)
					{
						//Something went wrong restoring state.
						//Assume there is no state and continue
					}
				}
			}

			frame.Navigate(typeof(SearchResultsPage), args.QueryText);
			Window.Current.Content = frame;

			// Ensure the current window is active
			Window.Current.Activate();
		}

		protected override void OnWindowCreated(WindowCreatedEventArgs args)
		{
			var searchPane = SearchPane.GetForCurrentView();
			searchPane.SuggestionsRequested += searchPane_SuggestionsRequested;
			searchPane.ResultSuggestionChosen += searchPane_ResultSuggestionChosen;
			searchPane.ShowOnKeyboardInput = true;
			base.OnWindowCreated(args);
		}

		async void searchPane_SuggestionsRequested(SearchPane sender, SearchPaneSuggestionsRequestedEventArgs args)
		{
			IList<string> result = (from tag in SampleDataSource.Tags where tag.StartsWith(args.QueryText.ToLower()) orderby tag select tag).ToList();
			args.Request.SearchSuggestionCollection.AppendQuerySuggestions(result);

			if(result.Count > 0)
			{
				string topTag = result.ElementAt(0);

				var deferral = args.Request.GetDeferral();

				try
				{
					args.Request.SearchSuggestionCollection.AppendSearchSeparator("Recommended");
					await SampleDataSource.Search(topTag);

					SampleDataGroup group = SampleDataSource.GetGroup("Search");
					if(group != null)
					{
						SampleDataItem item = group.Items[0];
						RandomAccessStreamReference stream = RandomAccessStreamReference.CreateFromUri(new Uri(item.ImagePath));
						args.Request.SearchSuggestionCollection.AppendResultSuggestion(item.Title, item.Subtitle, item.UniqueId, stream, item.Title);				
					}
				}
				finally
				{
					deferral.Complete();
				}
			}
		}

		void searchPane_ResultSuggestionChosen(SearchPane sender, SearchPaneResultSuggestionChosenEventArgs args)
		{
			(Window.Current.Content as Frame).Navigate(typeof(ItemDetailPage), args.Tag);
		}

		/// <summary>
		/// Invoked when the application is activated to display a file open picker.
		/// </summary>
		/// <param name="args">Details about the activation request.</param>
		protected override void OnFileOpenPickerActivated(FileOpenPickerActivatedEventArgs args)
		{
			var fileOpenPickerPage = new FileOpenPickerPage();
			fileOpenPickerPage.Activate(args);
		}
	}
}
